/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "globus_gridftp_server.h"
#include "globus_gsi_authz.h"
#include "version.h"

#define FTP_SERVICE_NAME "file"

static
char * 
globus_gfs_acl_customgsiauthzinterface_action_to_string(
    globus_gfs_acl_action_t             action)
{
    switch(action)
    {
        case GFS_ACL_ACTION_DELETE:
            return "delete";
        case GFS_ACL_ACTION_WRITE:
            return "write";
        case GFS_ACL_ACTION_CREATE:
            return "create";
        case GFS_ACL_ACTION_READ:
            return "read";
        case GFS_ACL_ACTION_LOOKUP:
            return "lookup";
        case GFS_ACL_ACTION_AUTHZ_ASSERT:
            return "authz_assert";
        default:
            return NULL;
    }
}

static
void
globus_gfs_acl_customgsiauthzinterface_cb(
    void *                              callback_arg,
    globus_gsi_authz_handle_t           handle,
    globus_result_t                     result)
{
    globus_gfs_acl_handle_t             acl_handle;
    GlobusGFSName(globus_gfs_acl_customgsiauthzinterface_cb);
    GlobusGFSDebugEnter();

    acl_handle = (globus_gfs_acl_handle_t) callback_arg;
    globus_gfs_acl_authorized_finished(acl_handle, result);

    GlobusGFSDebugExit();
}

static
int
globus_gfs_acl_customgsiauthzinterface_init(
    void **                             out_handle,
    globus_gfs_acl_info_t *             acl_info,
    globus_gfs_acl_handle_t             acl_handle,
    globus_result_t *                   out_res)
{
    globus_gsi_authz_handle_t           authz_handle;
    GlobusGFSName(globus_gfs_acl_customgsiauthzinterface_init);
    GlobusGFSDebugEnter();

    if(acl_info->context == NULL)
    {
        goto err;
    }
    *out_res = globus_gsi_authz_handle_init(
        &authz_handle,
        FTP_SERVICE_NAME,
        acl_info->context,
        globus_gfs_acl_customgsiauthzinterface_cb,
        acl_handle);
    if(*out_res != GLOBUS_SUCCESS)
    {
        goto err;
    }
    *out_handle = authz_handle;
    GlobusGFSDebugExit();
    return GLOBUS_GFS_ACL_WOULD_BLOCK;

  err:
    GlobusGFSDebugExitWithError();
    return GLOBUS_GFS_ACL_COMPLETE;
}

static
int
globus_gfs_acl_customgsiauthzinterface_authorize(
    void *                              out_handle,
    globus_gfs_acl_action_t             action,
    globus_gfs_acl_object_desc_t *      object,
    globus_gfs_acl_info_t *             acl_info,
    globus_gfs_acl_handle_t             acl_handle,
    globus_result_t *                   out_res)
{
    globus_gsi_authz_handle_t           authz_handle;
    char *                              full_object;
    char *                              action_str;
    GlobusGFSName(globus_gfs_acl_customgsiauthzinterface_authorize);
    GlobusGFSDebugEnter();

    authz_handle = (globus_gsi_authz_handle_t) out_handle;
    if(acl_info->context == NULL)
    {
        goto err;
    }
    /*
     * If the action is "authz_assert" then the object contains the assertions
     * received over the gridftp control channel - just pass it unmodified to
     * the authz callout
     */
    if(action == GFS_ACL_ACTION_AUTHZ_ASSERT)
    {
        full_object = globus_libc_strdup(object->name);
    }
    else
    {
        full_object = globus_common_create_string(
           "gsiftp://%s:%d%s", 
           acl_info->hostname, 
           globus_gfs_config_get_int("port"), 
           object->name);
    }    

    action_str = globus_gfs_acl_customgsiauthzinterface_action_to_string(action);

    *out_res = globus_gsi_authorize(
        authz_handle,
        action_str,
        full_object,
        globus_gfs_acl_customgsiauthzinterface_cb,
        acl_handle);
    globus_free(full_object);
    if(*out_res != GLOBUS_SUCCESS)
    {
        goto err;
    }

    GlobusGFSDebugExit();
    return GLOBUS_GFS_ACL_WOULD_BLOCK;

  err:
    GlobusGFSDebugExitWithError();
    return GLOBUS_GFS_ACL_COMPLETE;
}

static
void
globus_gfs_acl_customgsiauthzinterface_destroy_cb(
    void *                              callback_arg,
    globus_gsi_authz_handle_t           handle,
    globus_result_t                     result)
{
    GlobusGFSName(globus_gfs_acl_customgsiauthzinterface_destroy_cb);
    GlobusGFSDebugEnter();


    GlobusGFSDebugExit();
}

static
void
globus_gfs_acl_customgsiauthzinterface_destroy(
    void *                              out_handle)
{
    globus_gsi_authz_handle_t           authz_handle;
    GlobusGFSName(globus_gfs_acl_customgsiauthzinterface_destroy);
    GlobusGFSDebugEnter();

    authz_handle = (globus_gsi_authz_handle_t) out_handle;
    globus_gsi_authz_handle_destroy(
        authz_handle, globus_gfs_acl_customgsiauthzinterface_destroy_cb, NULL);

    GlobusGFSDebugExit();
}

/* module activation and registration calls */

/* plugin specific module descriptor.  
 * Only define the functions you implement, otherwise NULL */
globus_gfs_acl_module_t                 globus_gfs_acl_customgsiauthzinterface_module = 
{
    globus_gfs_acl_customgsiauthzinterface_init,
    globus_gfs_acl_customgsiauthzinterface_authorize,
    globus_gfs_acl_customgsiauthzinterface_destroy,
    NULL
};

static int globus_gfs_acl_customgsiauthzinterface_activate(void);
static int globus_gfs_acl_customgsiauthzinterface_deactivate(void);

/* globus module system descriptor */ 
GlobusExtensionDefineModule(globus_gridftp_server_acl_customgsiauthzinterface) =
{
    "globus_gridftp_server_acl_customgsiauthzinterface",
    globus_gfs_acl_customgsiauthzinterface_activate,
    globus_gfs_acl_customgsiauthzinterface_deactivate,
    NULL,
    NULL,
    &local_version
};


/* activation gets called when the module is loaded.  All we need to do here
 * is insert ourselves into the gridftp acl registry, and remove when we
 * get unloaded. */
static
int
globus_gfs_acl_customgsiauthzinterface_activate(void)
{
    globus_extension_registry_add(
        GLOBUS_GFS_ACL_REGISTRY,
        "customgsiauthzinterface",
        GlobusExtensionMyModule(globus_gridftp_server_acl_customgsiauthzinterface),
        &globus_gfs_acl_customgsiauthzinterface_module);

    return 0;
}

static
int
globus_gfs_acl_customgsiauthzinterface_deactivate(void)
{
    globus_extension_registry_remove(
        GLOBUS_GFS_ACL_REGISTRY, "customgsiauthzinterface");

    return 0;
}
